VERSION 5.00
Begin VB.UserControl MLListBox 
   BorderStyle     =   1  'Fixed Single
   ClientHeight    =   1785
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2595
   Picture         =   "MLListBox.ctx":0000
   ScaleHeight     =   119
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   173
   ToolboxBitmap   =   "MLListBox.ctx":0102
End
Attribute VB_Name = "MLListBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "axMultiLine ListBox"
'*********************************************************************************************
'
' MultiLine ListBox
'
' Main UserControl module
'
'*********************************************************************************************
'
' Author: Eduardo Morcillo
' E-Mail: edanmo@geocities.com
' Web Page: http://www.domaindlx.com/e_morcillo
'
' Created: 07/29/1999
' Updates:
'
'    07/31/1999. * Added item images.
'    01/04/2000. * Changed itemdata storage. Now
'                  is stored using LocallAlloc instead
'                  of an array.
'                * Added SelectedImage property.
'                * New events: Scroll, KeyUp, KeyDown,
'                  KeyPress
'
'*********************************************************************************************
Option Explicit

Dim m_OldWinProc As Long
Dim m_LBWinProc As Long
Dim m_IsHooked As Boolean
Dim m_LB As Long
Dim m_BackBrush As Long

'Default Property Values:
Const m_def_ShowBar = True
Const m_def_WordWrap = True
Const m_def_Padding = 0
Const m_def_Alignment = vbLeftJustify
Const m_def_BackColor = vbWindowBackground
Const m_def_ForeColor = vbWindowText
Const m_def_hWnd = 0

'Property Variables:
Dim m_ShowBar As Boolean
Dim m_WordWrap As Boolean
Dim m_Padding As Long
Dim m_Alignment As VBRUN.AlignmentConstants
Dim m_BackColor As OLE_COLOR
Dim WithEvents m_Font As StdFont
Attribute m_Font.VB_VarHelpID = -1
Dim m_ForeColor As OLE_COLOR

Event Click()
Attribute Click.VB_UserMemId = -600
Event DblClick()
Attribute DblClick.VB_UserMemId = -601
Event KeyDown(KeyCode As Integer, Shift As Integer)
Attribute KeyDown.VB_UserMemId = -602
Event KeyPress(KeyAscii As Integer)
Attribute KeyPress.VB_UserMemId = -603
Event KeyUp(KeyCode As Integer, Shift As Integer)
Attribute KeyUp.VB_UserMemId = -604
Event Scroll()

'*********************************************************************************************
' Adds an item
'*********************************************************************************************
Public Function AddItem(ByVal Text As String, Optional ByVal Index As Variant, Optional ByVal Image As StdPicture, Optional ByVal SelectedImage As StdPicture) As Integer
Dim ItemData As Long

    If IsMissing(Index) Or IsEmpty(Index) Then
        
        AddItem = SendMessage(m_LB, LB_ADDSTRING, 0, ByVal Text)
        
    Else
        
        AddItem = SendMessage(m_LB, LB_INSERTSTRING, Index, ByVal Text)
            
    End If

    If AddItem < 0 Then
        Err.Raise 381
    Else
        
        ' Create a new ItemData item
        ItemData = pCreateItemData(0, Image, SelectedImage)
        
        ' Set the listbox itemdata.
        SendMessage m_LB, LB_SETITEMDATA, AddItem, ByVal ItemData
        
        If Not Image Is Nothing Then
            ' Update item height
            SendMessage m_LB, LB_SETITEMHEIGHT, AddItem, ByVal pvItemHeight(AddItem, ItemData)
        End If
        
    End If

End Function

Public Property Get ItemData(ByVal Index As Integer) As Long
Dim IDataPtr As Long

    IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
    If IDataPtr = -1 Then
        Err.Raise 381
    Else
        ItemData = pGetItemDataData(IDataPtr)
    End If
     
End Property

Public Property Let ItemData(ByVal Index As Integer, ByVal Data As Long)
Dim IDataPtr As Long

    IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
    If IDataPtr = -1 Then
        Err.Raise 381
    Else
        pSetItemDataData IDataPtr, Data
    End If
    
End Property

Public Property Get ItemHeight(ByVal Index As Integer) As Long

    ItemHeight = SendMessage(m_LB, LB_GETITEMHEIGHT, Index, 0)
    
    If ItemHeight < 0 Then Err.Raise 381
    
End Property

Public Property Let ItemHeight(ByVal Index As Integer, ByVal New_Height As Long)
Dim R As Long

    R = SendMessage(m_LB, LB_SETITEMHEIGHT, Index, ByVal New_Height)
    
    If R < 0 Then Err.Raise 381
    
    RedrawWindow m_LB, ByVal 0&, 0, RDW_INVALIDATE
    
End Property

Public Property Get List(ByVal Index As Integer) As String
Attribute List.VB_MemberFlags = "400"
Dim R As Long

    R = SendMessage(m_LB, LB_GETTEXTLEN, Index, 0)
     
    If R < 0 Then Err.Raise 381
    
    List = String(R, 0)
    SendMessage m_LB, LB_GETTEXT, Index, ByVal List
    
End Property

Public Property Let List(ByVal Index As Integer, ByVal New_Text As String)
Dim R As Long, Idx As Long
    
    ' Insert a new item
    R = SendMessage(m_LB, LB_INSERTSTRING, Index, ByVal New_Text)
    If R < 0 Then Err.Raise 381
    
    ' Get ItemData index...
    Idx = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    ' and put it in the new item
    SendMessage m_LB, LB_SETITEMDATA, R, ByVal Idx
    
    ' Delete the old item
    SendMessage m_LB, LB_DELETESTRING, Index + 1, 0
    
End Property

Public Property Get ListCount()
Attribute ListCount.VB_MemberFlags = "400"

    ListCount = SendMessage(m_LB, LB_GETCOUNT, 0, 0)
         
End Property

Public Property Get ListIndex() As String
Attribute ListIndex.VB_MemberFlags = "400"

    ListIndex = SendMessage(m_LB, LB_GETCURSEL, 0, 0)
    
End Property

Public Property Get Picture(ByVal Index As Long) As StdPicture
Attribute Picture.VB_MemberFlags = "400"
Dim IDataPtr As Long

   IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
   If IDataPtr = -1 Then
      Err.Raise 381
   Else
      Set Picture = pGetItemDataImage(IDataPtr)
   End If
    
End Property

Public Property Get SelectedPicture(ByVal Index As Long) As StdPicture
Attribute SelectedPicture.VB_MemberFlags = "400"
Dim IDataPtr As Long

   IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
   If IDataPtr = -1 Then
      Err.Raise 381
   Else
      Set SelectedPicture = pGetItemDataImage(IDataPtr, itSelected)
   End If
    
End Property


Public Property Set Picture(ByVal Index As Long, ByVal New_Image As StdPicture)
Dim IDataPtr As Long

   IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
   If IDataPtr = -1 Then
      Err.Raise 381
   Else
    
      pSetItemDataImage IDataPtr, New_Image
    
      SendMessage m_LB, LB_SETITEMHEIGHT, Index, ByVal pvItemHeight(Index, IDataPtr)
    
      RedrawWindow m_LB, ByVal 0&, 0, RDW_INVALIDATE
      
   End If
    
End Property

Public Property Set SelectedPicture(ByVal Index As Long, ByVal New_Image As StdPicture)
Dim IDataPtr As Long

   IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, 0)
    
   If IDataPtr = -1 Then
      Err.Raise 381
   Else
    
      pSetItemDataImage IDataPtr, New_Image, itSelected
    
      SendMessage m_LB, LB_SETITEMHEIGHT, Index, ByVal pvItemHeight(Index, IDataPtr)
    
      RedrawWindow m_LB, ByVal 0&, 0, RDW_INVALIDATE
      
   End If
    
End Property


Private Function pvCreateListBox()
Dim Style As Long, ExStyle As Long

    Style = LBS_HASSTRINGS Or LBS_NOINTEGRALHEIGHT Or _
            LBS_OWNERDRAWVARIABLE Or LBS_NOTIFY Or _
            WS_VISIBLE Or WS_CHILD Or WS_GROUP Or _
            WS_TABSTOP Or WS_VSCROLL
    
    If Ambient.RightToLeft Then
        ExStyle = ExStyle Or WS_EX_LEFTSCROLLBAR Or WS_EX_RTLREADING
    End If
    
    m_LB = CreateWindowEx(ExStyle, "ListBox", vbNullString, _
             Style, 0, 0, ScaleWidth, ScaleHeight, _
             UserControl.hwnd, 1, App.hInstance, ByVal 0&)
             
    If m_LB = 0 Then
        Err.Raise vbObjectError, , "Cannot create ListBox control"
    Else
    
        m_Font_FontChanged ""
                
        If Ambient.UserMode Then
        
            ' Subclass the listbox
            SetWindowLong m_LB, GWL_USERDATA, ObjPtr(Me)
            m_LBWinProc = SetWindowLong(m_LB, GWL_WNDPROC, AddressOf mdlWindowProc.WinProc)
            
        Else
             AddItem "Control Name:" & vbCrLf & Ambient.DisplayName, , UserControl.Picture
        End If
        
    End If
    
End Function

Private Function pvDestroyListBox()
Dim IDataPtr As Long, Idx As Long

   ' Unsubclass the listbox
   If m_LBWinProc <> 0 Then
       SetWindowLong m_LB, GWL_WNDPROC, m_LBWinProc
       m_LBWinProc = 0
   End If

   ' Destroy all ItemData UDTs
   For Idx = 0 To SendMessage(m_LB, LB_GETCOUNT, 0, ByVal 0&) - 1
      IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Idx, ByVal 0&)
      pDestroyItemData IDataPtr
   Next
   
   ' Destroy the listbox
   DestroyWindow m_LB
   m_LB = 0
    
End Function

Private Sub pvDrawItem(ByVal lParam As Long)
Dim DI As DRAWITEMSTRUCT, Text As String
Dim Fmt As Long, hBr As Long, IDataPtr As Long
Dim Img As IPicture, Rct As RECT

   On Error Resume Next
    
   ' Copy the DRAWITEMSTRUCT from
   ' lParam ptr
   CopyMemory DI, ByVal lParam, Len(DI)
       
   With DI
       
      ' Get ItemData
      IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, .ItemID, 0)
     
      If m_ShowBar Then
      
           ' Erase the bar background
           Rct = .rcItem
           Rct.Top = Rct.Bottom - 10
           FillRect .hdc, Rct, m_BackBrush
           
           .rcItem.Bottom = .rcItem.Bottom - 8
      
      End If
      
      ' Draw the selection
      If (DI.itemState And ODS_SELECTED) Then
          
         hBr = CreateSolidBrush(OleClr2ClrRef(vbHighlight))
         
         FillRect .hdc, .rcItem, hBr
         DeleteObject hBr
         
         SetTextColor .hdc, OleClr2ClrRef(vbHighlightText)
      
         Set Img = pGetItemDataImage(IDataPtr, itSelected)
      
      Else
          
         FillRect .hdc, .rcItem, m_BackBrush
         SetTextColor .hdc, OleClr2ClrRef(m_ForeColor)
          
         Set Img = pGetItemDataImage(IDataPtr)
      
      End If
      
      ' Draw focus
      If (DI.itemState And ODS_FOCUS) Then DrawFocusRect .hdc, .rcItem
      
      If m_ShowBar Then
          
          .rcItem.Bottom = .rcItem.Bottom + 8
      
          Rct = .rcItem
          
          With Rct
              .Left = .Left + 8
              .Right = .Right - 8
              .Top = .Bottom - 5
          End With
          
          DrawEdge .hdc, Rct, BDR_SUNKENOUTER Or BDR_RAISEDINNER, BF_TOP
      
      End If
      
      If Not Img Is Nothing Then
          Dim TmpDC As Long, ScrDC As Long, Old As Long, Tp As Long
          
          Tp = .rcItem.Top + ((.rcItem.Bottom - .rcItem.Top + (10 * m_ShowBar)) - (Img.Height / 26.45)) / 2 + 2
      
          If Img.Type = vbPicTypeIcon Then
              
              DrawIconEx .hdc, .rcItem.Left + m_Padding, Tp, Img.Handle, Img.Width / 26.45, Img.Height / 26.45, 0, 0, 3
              
          ElseIf Img.Type = vbPicTypeBitmap Then
          
              ScrDC = GetDC(0&)
              TmpDC = CreateCompatibleDC(ScrDC)
              ReleaseDC 0&, ScrDC
                      
              Old = SelectObject(TmpDC, Img.Handle)
                                          
              BitBlt .hdc, .rcItem.Left + m_Padding, Tp, Img.Width / 26.45, Img.Height / 26.45, TmpDC, 0, 0, vbSrcCopy
          
              SelectObject TmpDC, Old
              DeleteDC 0&
          
          End If
          
          .rcItem.Left = .rcItem.Left + (Img.Width / 26.45) + m_Padding
          
      End If
      
      Text = String(SendMessage(m_LB, LB_GETTEXTLEN, .ItemID, 0), 0)
      SendMessage m_LB, LB_GETTEXT, .ItemID, ByVal Text
      
      Fmt = DT_EXPANDTABS Or DT_NOPREFIX
      
      If m_WordWrap Then Fmt = Fmt Or DT_WORDBREAK
            
      If Ambient.RightToLeft Then
          Fmt = Fmt Or DT_RTLREADING
          
          Select Case m_Alignment
              Case vbLeftJustify
                  Fmt = Fmt Or DT_RIGHT
              Case vbRightJustify
                  Fmt = Fmt Or DT_LEFT
              Case vbCenter
                  Fmt = Fmt Or DT_CENTER
          End Select
          
      Else
      
          Select Case m_Alignment
              Case vbLeftJustify
                  Fmt = Fmt Or DT_LEFT
              Case vbRightJustify
                  Fmt = Fmt Or DT_RIGHT
              Case vbCenter
                  Fmt = Fmt Or DT_CENTER
          End Select
          
      End If
      
      .rcItem.Left = .rcItem.Left + m_Padding
      .rcItem.Right = .rcItem.Right - m_Padding
      .rcItem.Top = .rcItem.Top + m_Padding
      
      Rct = .rcItem
      
      SetBkMode .hdc, NEWTRANSPARENT
      
      DrawText .hdc, Text, Len(Text), Rct, Fmt Or DT_CALCRECT
              
      .rcItem.Top = .rcItem.Top + ((.rcItem.Bottom - .rcItem.Top) - (Rct.Bottom - Rct.Top) + (10 * m_ShowBar)) / 2
      
      DrawText .hdc, Text, Len(Text), .rcItem, Fmt
       
   End With
    
End Sub

Private Sub pvMeasureItem(ByVal lParam As Long)
Dim MI As MEASUREITEMSTRUCT

    On Error Resume Next
    
    ' Copy the MEASUREITEMSTRUCT from
    ' lParam ptr
    CopyMemory MI, ByVal lParam, Len(MI)
       
    With MI
        
        .ItemHeight = pvItemHeight(.ItemID, -1)
        .ItemWidth = ScaleWidth
        
    End With
    
    ' Copy back the MEASUREITEMSTRUCT
    CopyMemory ByVal lParam, MI, Len(MI)
    
End Sub


Private Function pvItemHeight(ByVal Index As Long, ByVal ID As Long) As Long
Dim RequiredH As Long, Text As String
Dim Fnt As IFont, OldFnt As Long, Idx As Long
Dim DC As Long, R As RECT, Fmt As Long

    On Error Resume Next
            
    Set Fnt = m_Font
        
    Text = String(SendMessage(m_LB, LB_GETTEXTLEN, Index, 0), 0)
    SendMessage m_LB, LB_GETTEXT, Index, ByVal Text
    
    DC = GetDC(m_LB)
    
    OldFnt = SelectObject(DC, Fnt.hFont)
    
    Fmt = DT_CALCRECT Or DT_EXPANDTABS Or DT_LEFT Or _
          DT_NOPREFIX
          
    If m_WordWrap Then Fmt = Fmt Or DT_WORDBREAK
          
    R.Right = ScaleWidth - m_Padding * 2
    R.Bottom = &HFFFFFFF
    R.Left = m_Padding
    
    DrawText DC, Text, Len(Text), R, Fmt
            
    pvItemHeight = R.Bottom
    
    SelectObject DC, OldFnt
    ReleaseDC m_LB, DC

    If ID <> -1 Then
    
        RequiredH = pGetItemDataImage(ID).Height / 26.45
        If pvItemHeight < RequiredH Then pvItemHeight = RequiredH
    
    End If
    
    If m_ShowBar Then pvItemHeight = pvItemHeight + 10
    pvItemHeight = pvItemHeight + Padding * 2

End Function

Public Function RemoveItem(ByVal Index As Long)
Dim IDataPtr As Long

   IDataPtr = SendMessage(m_LB, LB_GETITEMDATA, Index, ByVal 0&)
   
   If IDataPtr = 1 Then
   
      Err.Raise 381
      
   Else
      
      pDestroyItemData IDataPtr
      
      SendMessage m_LB, LB_DELETESTRING, Index, ByVal 0&
      
   End If
   
End Function

Friend Function WindowProc(ByVal Wnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Attribute WindowProc.VB_MemberFlags = "40"
Static Focusing As Boolean

   ' This function is used for both the ListBox
   ' window proc and the control proc

   If Wnd = m_LB Then
   
      ' Process ListBox messages
      
      Select Case uMsg
            
         Case WM_SETFOCUS
         
            ' The following code forces VB
            ' to call GotFocus event
            ' when the ListBox receives the
            ' focus.
            
            Focusing = True
            
            SendMessage UserControl.hwnd, &H100E, 35, 0&
            SendMessage UserControl.hwnd, WM_SETFOCUS, wParam, 0&
            SendMessage UserControl.hwnd, &H100E, 14, 0&
            SendMessage UserControl.hwnd, WM_KILLFOCUS, Wnd, 0&
            SendMessage UserControl.hwnd, &H100E, 18, 0&
            SendMessage UserControl.hwnd, &H100E, 36, 0&
            
            Focusing = False
         
      End Select
      
      ' Call the previous window procedure
      WindowProc = CallWindowProc(m_LBWinProc, Wnd, uMsg, wParam, lParam)
   
      Select Case uMsg
         Case WM_VSCROLL
            RaiseEvent Scroll
      End Select
      
   Else

      Select Case uMsg
      
         Case WM_CTLCOLORLISTBOX ' Set background brush
            
            ' Delete previous brus          h
            DeleteObject m_BackBrush
            
            ' Create a new brush
            m_BackBrush = CreateSolidBrush(OleClr2ClrRef(m_BackColor))
            
            ' Return the new brush to windows
            WindowProc = m_BackBrush
            
         Case WM_DRAWITEM ' Draw an item
         
            pvDrawItem lParam
            
         Case WM_MEASUREITEM ' Measure an item
         
            pvMeasureItem lParam
      
         Case WM_COMMAND
      
            Select Case wParam \ &H10000
               Case LBN_DBLCLK
                  RaiseEvent DblClick
               Case LBN_SELCHANGE
                  RaiseEvent Click
            End Select
      
         Case WM_SETFOCUS
            
            If Not Focusing Then
               
               Focusing = True
               
               ' Set the focus to the listbox
               SetFocusAPI m_LB
               
               Focusing = False
               
            End If
            
            WindowProc = CallWindowProc(m_OldWinProc, Wnd, uMsg, wParam, lParam)
         
         Case WM_KILLFOCUS
         
            ' Kill the listbox focus
            SendMessage m_LB, WM_KILLFOCUS, wParam, 0
            
            WindowProc = CallWindowProc(m_OldWinProc, Wnd, uMsg, wParam, lParam)
            
         Case Else
         
            WindowProc = CallWindowProc(m_OldWinProc, Wnd, uMsg, wParam, lParam)
      
      End Select
   
   End If
   
End Function

Private Sub UserControl_KeyDown(KeyCode As Integer, Shift As Integer)

   RaiseEvent KeyDown(KeyCode, Shift)
    
   If KeyCode Then
      ' Map KeyDown to ListBox
      CallWindowProc m_LBWinProc, m_LB, WM_KEYDOWN, KeyCode, 0
   End If
   
End Sub

Private Sub UserControl_KeyPress(KeyAscii As Integer)

   RaiseEvent KeyPress(KeyAscii)
   
   If KeyAscii Then
      ' Map KeyPress to ListBox
      CallWindowProc m_LBWinProc, m_LB, WM_CHAR, KeyAscii, 0
   End If
   
End Sub

Private Sub UserControl_KeyUp(KeyCode As Integer, Shift As Integer)

   RaiseEvent KeyUp(KeyCode, Shift)
   
   If KeyCode Then
      ' Map KeyUp to ListBox
      Call CallWindowProc(m_LBWinProc, m_LB, WM_KEYUP, KeyCode, 0)
   End If
   
End Sub

Private Sub pvHook()
    
    If Not m_IsHooked Then
        
        ' Set control reference
        SetWindowLong UserControl.hwnd, GWL_USERDATA, ObjPtr(Me)
        
        ' Subclass the user control
        m_OldWinProc = SetWindowLong(UserControl.hwnd, GWL_WNDPROC, AddressOf mdlWindowProc.WinProc)
                
        ' Mark the ctrl as subclassed
        m_IsHooked = True
        
    End If
    
End Sub

Private Sub pvUnHook()

    If m_IsHooked Then
        
        ' Unsubclass ctrl
        SetWindowLong UserControl.hwnd, GWL_WNDPROC, m_OldWinProc
        SetWindowLong UserControl.hwnd, GWL_USERDATA, 0&
        
        m_IsHooked = False
        
    End If
    
End Sub

Private Sub m_Font_FontChanged(ByVal PropertyName As String)
Dim IFnt As IFont

    ' Update listbox font
    Set IFnt = m_Font
    
    SendMessage m_LB, WM_SETFONT, IFnt.hFont, 0
    
    Set IFnt = Nothing
    
End Sub

Private Sub UserControl_Initialize()

    pvHook
    
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    
    m_BackColor = PropBag.ReadProperty("BackColor", m_def_BackColor)
    Set m_Font = PropBag.ReadProperty("Font", Ambient.Font)
    m_ForeColor = PropBag.ReadProperty("ForeColor", m_def_ForeColor)
    m_Alignment = PropBag.ReadProperty("Alignment", m_def_Alignment)
    m_WordWrap = PropBag.ReadProperty("WordWrap", m_def_WordWrap)
    m_Padding = PropBag.ReadProperty("Padding", m_def_Padding)
    m_ShowBar = PropBag.ReadProperty("ShowBar", m_def_ShowBar)

    pvCreateListBox

End Sub

Private Sub UserControl_Resize()

    SetWindowPos m_LB, 0, 0, 0, ScaleWidth, ScaleHeight, SWP_NOMOVE Or SWP_NOZORDER
    
    RedrawWindow m_LB, ByVal 0&, 0&, RDW_INVALIDATE Or RDW_NOERASE
    
End Sub

Private Sub UserControl_Terminate()

    pvDestroyListBox
    pvUnHook
    
End Sub
Public Property Get BackColor() As OLE_COLOR
Attribute BackColor.VB_Description = "Returns/sets the background color used to display text and graphics in an object."
Attribute BackColor.VB_UserMemId = -501
    BackColor = m_BackColor
End Property

Public Property Let BackColor(ByVal New_BackColor As OLE_COLOR)
    
    m_BackColor = New_BackColor
    PropertyChanged "BackColor"
        
    SetWindowPos m_LB, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOZORDER
    UserControl_Resize
    
End Property

Public Property Get Font() As Font
Attribute Font.VB_Description = "Returns a Font object."
Attribute Font.VB_UserMemId = -512
    Set Font = m_Font
End Property

Public Property Set Font(ByVal New_Font As Font)
    
    Set m_Font = New_Font
    PropertyChanged "Font"
    
    m_Font_FontChanged ""
    
End Property

Public Property Get ForeColor() As OLE_COLOR
Attribute ForeColor.VB_Description = "Returns/sets the foreground color used to display text and graphics in an object."
Attribute ForeColor.VB_UserMemId = -513
    ForeColor = m_ForeColor
End Property

Public Property Let ForeColor(ByVal New_ForeColor As OLE_COLOR)
    m_ForeColor = New_ForeColor
    PropertyChanged "ForeColor"
End Property

Public Property Get hwnd() As Long
Attribute hwnd.VB_Description = "Returns a handle (from Microsoft Windows) to an object's window."
Attribute hwnd.VB_UserMemId = -515
    hwnd = m_LB
End Property

'Initialize Properties for User Control
Private Sub UserControl_InitProperties()
    
    m_BackColor = m_def_BackColor
    Set m_Font = Ambient.Font
    m_ForeColor = m_def_ForeColor
    m_Alignment = m_def_Alignment
    m_WordWrap = m_def_WordWrap
    m_Padding = m_def_Padding
    m_ShowBar = m_def_ShowBar

    pvCreateListBox

End Sub

'Write property values to storage
Private Sub UserControl_WriteProperties(PropBag As PropertyBag)

    Call PropBag.WriteProperty("BackColor", m_BackColor, m_def_BackColor)
    Call PropBag.WriteProperty("Font", m_Font, Ambient.Font)
    Call PropBag.WriteProperty("ForeColor", m_ForeColor, m_def_ForeColor)
    Call PropBag.WriteProperty("Alignment", m_Alignment, m_def_Alignment)
    Call PropBag.WriteProperty("WordWrap", m_WordWrap, m_def_WordWrap)
    Call PropBag.WriteProperty("Padding", m_Padding, m_def_Padding)
    Call PropBag.WriteProperty("ShowBar", m_ShowBar, m_def_ShowBar)
End Sub

Public Property Get Alignment() As VBRUN.AlignmentConstants
    Alignment = m_Alignment
End Property

Public Property Let Alignment(ByVal New_Alignment As VBRUN.AlignmentConstants)
    
    m_Alignment = New_Alignment
    PropertyChanged "Alignment"
    
    RedrawWindow m_LB, ByVal 0&, 0, RDW_INVALIDATE
End Property

Public Property Get WordWrap() As Boolean
    WordWrap = m_WordWrap
End Property

Public Property Let WordWrap(ByVal New_WordWrap As Boolean)
    
    If Ambient.UserMode Then Err.Raise 382
    
    m_WordWrap = New_WordWrap
    PropertyChanged "WordWrap"
    
End Property

Public Property Get Padding() As Long
    Padding = m_Padding
End Property

Public Property Let Padding(ByVal New_Padding As Long)
    
    If Ambient.UserMode Then Err.Raise 382
    
    m_Padding = New_Padding
    PropertyChanged "Padding"
    
    SendMessage m_LB, LB_SETITEMHEIGHT, 0, ByVal pvItemHeight(0, 0)
    SetWindowPos m_LB, 0, 0, 0, 0, 0, SWP_NOZORDER Or SWP_NOMOVE
    UserControl_Resize
    
End Property

Public Property Get ShowBar() As Boolean
    ShowBar = m_ShowBar
End Property

Public Property Let ShowBar(ByVal New_ShowBar As Boolean)
    
    If Ambient.UserMode Then Err.Raise 382
    
    m_ShowBar = New_ShowBar
    PropertyChanged "ShowBar"
    
    If m_ShowBar Then
        ItemHeight(0) = ItemHeight(0) + 10
    Else
        ItemHeight(0) = ItemHeight(0) - 10
    End If
    
    SetWindowPos m_LB, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOZORDER
    UserControl_Resize

End Property

