Attribute VB_Name = "mdlItemData"
'*********************************************************************************************
'
' MultiLine ListBox
'
' ItemData support functions module
'
'*********************************************************************************************
'
' Author: Eduardo Morcillo
' E-Mail: edanmo@geocities.com
' Web Page: http://www.domaindlx.com/e_morcillo
'
' Created: 01/04/2000
'
'*********************************************************************************************
Option Explicit

Enum ImageType
   itNormal
   itSelected
End Enum

' Non object data. Use a different
' struct for non object members
' so when you get/set them the
' objects are not released by error.
Private Type NonObjectMembers
    ItemData As Long
End Type

' The complete ItemData struct
Private Type ItemDataUDT
   NonObj As NonObjectMembers
   Image As StdPicture
   SelectedImage As StdPicture
End Type

Public Function pCreateItemData(ByVal ItemData As Long, ByVal Image As StdPicture, ByVal SelectedImage As StdPicture) As Long
Dim TmpUDT As ItemDataUDT

   ' Fill the temporary UDT
   With TmpUDT
      .NonObj.ItemData = ItemData
      Set .Image = Image
      Set .SelectedImage = SelectedImage
   End With
   
   ' Alloc local memory
   pCreateItemData = LocalAlloc(LMEM_FIXED, LenB(TmpUDT))
   
   ' Check if pCreateItemData is a
   ' valid pointer
   If pCreateItemData <> 0 Then
   
      ' Copy the UDT to the
      ' allocated memory
      CopyMemory ByVal pCreateItemData, TmpUDT, Len(TmpUDT)
      
      ' Remove the image from
      ' UDT without decrementing
      ' the reference count.
      ' The reference count will be
      ' decremented in pDestroyItemData.
      CopyMemory TmpUDT.Image, 0&, 4
      CopyMemory TmpUDT.SelectedImage, 0&, 4
      
   Else
      
      Err.Raise 7
      
   End If
   
End Function


Public Sub pDestroyItemData(ByVal ItemDataPtr As Long)
Dim TmpUDT As ItemDataUDT

   ' Check if ItemDataPtr is a
   ' valid pointer
   If ItemDataPtr <> 0 Then
   
      ' Copy the data from the
      ' pointer to the UDT
      CopyMemory TmpUDT, ByVal ItemDataPtr, Len(TmpUDT)
      
      ' Destroy the image. This
      ' call will decrement the
      ' reference count.
      Set TmpUDT.Image = Nothing
      Set TmpUDT.SelectedImage = Nothing
      
      ' Free the memory
      LocalFree ItemDataPtr
         
   End If
   
End Sub



Public Function pGetItemDataImage(ByVal ItemDataPtr As Long, Optional ByVal IType As ImageType = itNormal) As StdPicture
Dim TmpUDT As ItemDataUDT

   ' Check if ItemDataPtr is a
   ' valid pointer
   If ItemDataPtr <> 0 Then
   
      ' Copy the data from the
      ' pointer to the UDT
      CopyMemory TmpUDT, ByVal ItemDataPtr, Len(TmpUDT)
      
      If IType = itNormal Then
         
         ' Get the image from the
         ' UDT
         Set pGetItemDataImage = TmpUDT.Image
      
      Else
         
         ' Get the image from the
         ' UDT
         If TmpUDT.SelectedImage Is Nothing Then
            Set pGetItemDataImage = TmpUDT.Image
         Else
            Set pGetItemDataImage = TmpUDT.SelectedImage
         End If
      
      End If
      
      ' Set the images in the
      ' UDT to nothing
      CopyMemory TmpUDT.Image, 0&, 4
      CopyMemory TmpUDT.SelectedImage, 0&, 4
   
   End If

End Function


Public Sub pSetItemDataImage(ByVal ItemDataPtr As Long, ByVal Image As StdPicture, Optional IType As ImageType = itNormal)
Dim TmpUDT As ItemDataUDT

   ' Check if ItemDataPtr is a
   ' valid pointer
   If ItemDataPtr <> 0 Then
   
      ' Copy the data from the
      ' pointer to the UDT
      CopyMemory TmpUDT, ByVal ItemDataPtr, Len(TmpUDT)
      
      If IType = itNormal Then
         
         ' Decrement the reference
         ' count of previous image.
         Set TmpUDT.Image = Nothing
         
         ' Set the new image
         Set TmpUDT.Image = Image
            
      Else
         
         ' Decrement the reference
         ' count of previous image.
         Set TmpUDT.SelectedImage = Nothing
         
         ' Set the new image
         Set TmpUDT.SelectedImage = Image
      
      End If
      
      ' Copy back the UDT to
      ' the memory pointed by
      ' ItemDataPtr
      CopyMemory ByVal ItemDataPtr, TmpUDT, Len(TmpUDT)
      
      ' Set the images in the
      ' UDT to nothing
      CopyMemory TmpUDT.Image, 0&, 4
      CopyMemory TmpUDT.SelectedImage, 0&, 4
      
   End If

End Sub



Public Sub pSetItemDataData(ByVal ItemDataPtr As Long, ByVal IData As Long)
Dim TmpUDT As NonObjectMembers

   ' Check if ItemDataPtr is a
   ' valid pointer
   If ItemDataPtr <> 0 Then
   
      ' Copy the data from the
      ' pointer to the UDT
      CopyMemory TmpUDT, ByVal ItemDataPtr, Len(TmpUDT)
      
      ' Set the data
      TmpUDT.ItemData = IData
      
      ' Copy back the UDT to
      ' the memory pointed by
      ' ItemDataPtr
      CopyMemory ByVal ItemDataPtr, TmpUDT, Len(TmpUDT)
      
   End If

End Sub

Public Function pGetItemDataData(ByVal ItemDataPtr As Long) As Long
Dim TmpUDT As NonObjectMembers

   ' Check if ItemDataPtr is a
   ' valid pointer
   If ItemDataPtr <> 0 Then
   
      ' Copy the data from the
      ' pointer to the UDT
      CopyMemory TmpUDT, ByVal ItemDataPtr, Len(TmpUDT)
      
      ' Get the long value from the UDT
      pGetItemDataData = TmpUDT.ItemData
      
   End If

End Function



